#pragma once

#include <Game/Core.hpp>
#include <bt/behavior_node.hpp>

namespace bt {
namespace meta {

template <typename T>
concept Interactable =
    std::same_as<T, Interactable::Item> || std::same_as<T, Interactable::NPC> ||
    std::same_as<T, Interactable::Player> ||
    std::same_as<T, Interactable::Projectile> ||
    std::same_as<T, Interactable::Widget> ||
    std::same_as<T, Interactable::Item>;
}

template <typename Context, typename ActionSource, typename InteractableSource>
  requires  std::is_same_v<std::string, ActionSource> ||
            std::is_same_v<std::string, std::invoke_result_t<ActionSource>> ||
            std::is_same_v<std::string, std::invoke_result_t<ActionSource, Context &>> &&
            meta::Interactable<std::invoke_result_t<InteractableSource>> ||
            meta::Interactable<std::invoke_result_t<InteractableSource, Context &>>
class interact_node : public behavior_node<Context> {
public:
  ActionSource action_source;
  InteractableSource interactable_source;

  interact_node(ActionSource action_source,
                InteractableSource interactable_source)
      : action_source(action_source), interactable_source(interactable_source) {}

  node_task tick(Context &ctx) noexcept override {
    const auto interactable = [&]() {
      if constexpr (std::is_invocable_v<InteractableSource>)
        return interactable_source();
      else if constexpr (std::is_invocable_v<InteractableSource, Context &>)
        return interactable_source(ctx);
    }();

    if (!interactable)
      co_return task_result::failure;

    const auto action = [&]() {
      if constexpr (std::is_invocable_v<ActionSource>)
        return action_source();
      else if constexpr (std::is_invocable_v<ActionSource, Context &>)
        return action_source(ctx);
      else if constexpr (std::is_same_v<std::string, ActionSource>)
        return action_source;
    }();

    if (!interactable.Interact(action))
      co_return task_result::failure;
    
    co_return task_result::success;
  }
};

} // namespace bt