#pragma once

#include <Game/Core.hpp>

#include <bt/behavior_node.hpp>

namespace bt {
template <typename Context, typename SpellSource> class select_spell_node;

template <typename Context>
class select_spell_node<Context, Magic::SPELL> : public behavior_node<Context> {
public:
  Magic::SPELL spell;

  select_spell_node(Magic::SPELL spell) : spell(spell) {}

  node_task tick(Context &) noexcept override {
    if (!Magic::SelectSpell(spell))
      co_return task_result::failure;

    co_return task_result::success;
  }
};

template <typename Context, typename SpellGetter>
  requires std::is_same_v<Magic::SPELL,
                          std::result_of_t<SpellGetter(Context &)>>
class select_spell_node<Context, SpellGetter> : public behavior_node<Context> {
public:
  SpellGetter spell_getter;

  select_spell_node(SpellGetter spell_getter) : spell_getter(spell_getter) {}

  node_task tick(Context &ctx) noexcept override {
    if (!Magic::SelectSpell(spell_getter(ctx)))
      co_return task_result::failure;

    co_return task_result::success;
  }
};

template <typename Context, typename SpellSource> class spell_selected_node;

template<typename Context>
class spell_selected_node<Context, Magic::SPELL>
    : public behavior_node<Context> {
public:
  Magic::SPELL spell;

  spell_selected_node(Magic::SPELL spell) : spell(spell) {}

  node_task tick(Context &) noexcept override {
    if (!Magic::IsSpellSelected(spell))
      co_return task_result::failure;

    co_return task_result::success;
  }
};

template <typename Context, typename SpellGetter>
  requires std::is_same_v<Magic::SPELL,
                          std::result_of_t<SpellGetter(Context &)>>
class spell_selected_node<Context, SpellGetter>
    : public behavior_node<Context> {
public:
  SpellGetter spell_getter;

  spell_selected_node(SpellGetter spell_getter) : spell_getter(spell_getter) {}

  node_task tick(Context &ctx) noexcept override {
    if (!Magic::IsSpellSelected(spell_getter(ctx)))
      co_return task_result::failure;

    co_return task_result::success;
  }
};

// template <typename Context>
// class cast_spell_node : public behavior_node<Context> {
// public:
//   Magic::SPELL spell;

//   cast_spell_node(Magic::SPELL spell) : spell(spell) {}

//   node_task tick(Context &ctx) noexcept override {
//     if (!Magic::CastSpell(spell))
//       co_return task_result::failure;

//     co_return task_result::success;
//   }
// };

template <typename Context, typename SpellSource>
class cast_spell_node;

template <typename Context>
class cast_spell_node<Context, Magic::SPELL> : public behavior_node<Context> {
public:
  Magic::SPELL spell;

  cast_spell_node(Magic::SPELL spell) : spell(spell) {}

  node_task tick(Context &) noexcept override {
    if (!Magic::CastSpell(spell))
      co_return task_result::failure;

    co_return task_result::success;
  }
};

template <typename Context, typename SpellGetter>
  requires std::is_same_v<Magic::SPELL,
                          std::result_of_t<SpellGetter(Context &)>>
class cast_spell_node<Context, SpellGetter> : public behavior_node<Context> {
public:
  SpellGetter spell_getter;

  cast_spell_node(SpellGetter spell_getter) : spell_getter(spell_getter) {}

  node_task tick(Context &ctx) noexcept override {
    if (!Magic::CastSpell(spell_getter(ctx)))
      co_return task_result::failure;

    co_return task_result::success;
  }
};

} // namespace bt