#pragma once

#include <Game/Core.hpp>

#include <bt/behavior_node.hpp>
#include <bt/pathfinding.hpp>

#include <chrono>
#include <fstream>
#include <queue>
#include <unordered_map>
#include <unordered_set>
#include <variant>

namespace bt {
template <typename Context, typename DestinationSource, typename PFSSource>
  requires std::is_same_v<Tile, DestinationSource> ||
           std::is_same_v<Tile,
                          std::result_of_t<DestinationSource(Context &)>> ||
           std::is_same_v<pathfinding::settings, PFSSource> ||
           std::is_same_v<const pathfinding::settings &,
                          std::result_of_t<PFSSource(Context &)>>
class navigation_node : public behavior_node<Context> {
public:
  navigation_node(const DestinationSource &destination_source,
                  const PFSSource &pfs_source)
      : _destination_source(destination_source), _pfs_source(pfs_source) {}

  node_task tick(Context &ctx) noexcept override {
    const auto player = Players::GetLocal();
    if (!player)
      co_return task_result::failure;

    auto position = player.GetTile();
    if (!position)
      co_return task_result::failure;

    co_return task_result::success;
  }

private:
  DestinationSource _destination_source;
  PFSSource _pfs_source;
};
} // namespace bt