#pragma once

#include <bt/pathfinding.hpp>

namespace bt::pathfinding {
class game_object : public link {
public:
  Tile tile;
  std::string action;

  game_object(Tile tile, std::string action, Tile start,
              Tile destination) noexcept
      : link(start, destination), tile(tile), action(action) {}

  bool can_handle() const noexcept override { return true; }

  bool handle() noexcept override {
    auto player = Players::GetLocal();
    if (!player)
      return false;
    
    auto position = player.GetTile();
    if (!position)
      return false;
    
    if (position.DistanceFrom(tile) >= 5)
      return false;
    
    const auto obj = GameObjects::Get(tile);
    if (!obj)
      return false;
    
    if (!obj.Interact(action))
      return false;
    
    if (!WaitFunc(2000, 50, [&]() {
      position = player.GetTile();
      if (!position)
        return false;
      
      return position == destination;
    }))
      return false;
    
    return true;
  }
};

void load_obstacles() {
  add_link(std::make_shared<game_object>(Tile(3205, 3208, 0), "Climb-up",
                                             Tile(3205, 3208, 0),
                                             Tile(3205, 3209, 1)));
}
} // namespace bt::pathfinding